<?php
declare(strict_types=1);

namespace Inti\Provider;

use Inti\Provider\Query\QueryInterface;
use Inti\Provider\Query\QueryTrait;

use function array_filter;
use function count;

/**
 * Class AbstractProvider
 *
 * @package Inti\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
abstract class AbstractProvider implements
    ProviderInterface,
    QueryInterface
{
    use QueryTrait;

    protected abstract function fetchResults(): array;
    protected abstract function filterResult(array $result): array;

    public function getCount(): int
    {
        return count($this->getItems());
    }

    public function getItem(): ?array
    {
        return $this->getItems()[0] ?? null;
    }

    public function getItems(): array
    {
        $results = $this->fetchResults();
        return array_filter([$this, 'filterResult'], $results);
    }
}
